#
# Autogenerated by Thrift
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#  @generated
#

from __future__ import absolute_import
import six
import sys
from nebula2.fbthrift.util.Recursive import fix_spec
from nebula2.fbthrift.Thrift import TType, TMessageType, TPriority, TRequestContext, TProcessorEventHandler, TServerInterface, TProcessor, TException, TApplicationException, UnimplementedTypedef
from nebula2.fbthrift.protocol.TProtocol import TProtocolException


import nebula2.common.ttypes


import pprint
import warnings
from nebula2.fbthrift import Thrift
from nebula2.fbthrift.transport import TTransport
from nebula2.fbthrift.protocol import TBinaryProtocol
from nebula2.fbthrift.protocol import TCompactProtocol
from nebula2.fbthrift.protocol import THeaderProtocol
fastproto = None
try:
  from nebula2.fbthrift.protocol import fastproto
except ImportError:
  pass
all_structs = []
UTF8STRINGS = bool(0) or sys.version_info.major >= 3

__all__ = ['UTF8STRINGS', 'ProfilingStats', 'PlanNodeBranchInfo', 'Pair', 'PlanNodeDescription', 'PlanDescription', 'ExecutionResponse', 'AuthResponse', 'VerifyClientVersionResp', 'VerifyClientVersionReq']

class ProfilingStats:
  """
  Attributes:
   - rows
   - exec_duration_in_us
   - total_duration_in_us
   - other_stats
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I64:
          self.rows = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.exec_duration_in_us = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.total_duration_in_us = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.MAP:
          self.other_stats = {}
          (_ktype1, _vtype2, _size0 ) = iprot.readMapBegin() 
          if _size0 >= 0:
            for _i4 in six.moves.range(_size0):
              _key5 = iprot.readString()
              _val6 = iprot.readString()
              self.other_stats[_key5] = _val6
          else: 
            while iprot.peekMap():
              _key7 = iprot.readString()
              _val8 = iprot.readString()
              self.other_stats[_key7] = _val8
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('ProfilingStats')
    if self.rows != None:
      oprot.writeFieldBegin('rows', TType.I64, 1)
      oprot.writeI64(self.rows)
      oprot.writeFieldEnd()
    if self.exec_duration_in_us != None:
      oprot.writeFieldBegin('exec_duration_in_us', TType.I64, 2)
      oprot.writeI64(self.exec_duration_in_us)
      oprot.writeFieldEnd()
    if self.total_duration_in_us != None:
      oprot.writeFieldBegin('total_duration_in_us', TType.I64, 3)
      oprot.writeI64(self.total_duration_in_us)
      oprot.writeFieldEnd()
    if self.other_stats != None:
      oprot.writeFieldBegin('other_stats', TType.MAP, 4)
      oprot.writeMapBegin(TType.STRING, TType.STRING, len(self.other_stats))
      for kiter9,viter10 in self.other_stats.items():
        oprot.writeString(kiter9)
        oprot.writeString(viter10)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.rows is not None:
      value = pprint.pformat(self.rows, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    rows=%s' % (value))
    if self.exec_duration_in_us is not None:
      value = pprint.pformat(self.exec_duration_in_us, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    exec_duration_in_us=%s' % (value))
    if self.total_duration_in_us is not None:
      value = pprint.pformat(self.total_duration_in_us, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    total_duration_in_us=%s' % (value))
    if self.other_stats is not None:
      value = pprint.pformat(self.other_stats, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    other_stats=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class PlanNodeBranchInfo:
  """
  Attributes:
   - is_do_branch
   - condition_node_id
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.BOOL:
          self.is_do_branch = iprot.readBool()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.condition_node_id = iprot.readI64()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('PlanNodeBranchInfo')
    if self.is_do_branch != None:
      oprot.writeFieldBegin('is_do_branch', TType.BOOL, 1)
      oprot.writeBool(self.is_do_branch)
      oprot.writeFieldEnd()
    if self.condition_node_id != None:
      oprot.writeFieldBegin('condition_node_id', TType.I64, 2)
      oprot.writeI64(self.condition_node_id)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.is_do_branch is not None:
      value = pprint.pformat(self.is_do_branch, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    is_do_branch=%s' % (value))
    if self.condition_node_id is not None:
      value = pprint.pformat(self.condition_node_id, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    condition_node_id=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class Pair:
  """
  Attributes:
   - key
   - value
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.key = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.value = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('Pair')
    if self.key != None:
      oprot.writeFieldBegin('key', TType.STRING, 1)
      oprot.writeString(self.key)
      oprot.writeFieldEnd()
    if self.value != None:
      oprot.writeFieldBegin('value', TType.STRING, 2)
      oprot.writeString(self.value)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.key is not None:
      value = pprint.pformat(self.key, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    key=%s' % (value))
    if self.value is not None:
      value = pprint.pformat(self.value, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    value=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class PlanNodeDescription:
  """
  Attributes:
   - name
   - id
   - output_var
   - description
   - profiles
   - branch_info
   - dependencies
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.name = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I64:
          self.id = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.output_var = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.LIST:
          self.description = []
          (_etype14, _size11) = iprot.readListBegin()
          if _size11 >= 0:
            for _i15 in six.moves.range(_size11):
              _elem16 = Pair()
              _elem16.read(iprot)
              self.description.append(_elem16)
          else: 
            while iprot.peekList():
              _elem17 = Pair()
              _elem17.read(iprot)
              self.description.append(_elem17)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.LIST:
          self.profiles = []
          (_etype21, _size18) = iprot.readListBegin()
          if _size18 >= 0:
            for _i22 in six.moves.range(_size18):
              _elem23 = ProfilingStats()
              _elem23.read(iprot)
              self.profiles.append(_elem23)
          else: 
            while iprot.peekList():
              _elem24 = ProfilingStats()
              _elem24.read(iprot)
              self.profiles.append(_elem24)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRUCT:
          self.branch_info = PlanNodeBranchInfo()
          self.branch_info.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.LIST:
          self.dependencies = []
          (_etype28, _size25) = iprot.readListBegin()
          if _size25 >= 0:
            for _i29 in six.moves.range(_size25):
              _elem30 = iprot.readI64()
              self.dependencies.append(_elem30)
          else: 
            while iprot.peekList():
              _elem31 = iprot.readI64()
              self.dependencies.append(_elem31)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('PlanNodeDescription')
    if self.name != None:
      oprot.writeFieldBegin('name', TType.STRING, 1)
      oprot.writeString(self.name)
      oprot.writeFieldEnd()
    if self.id != None:
      oprot.writeFieldBegin('id', TType.I64, 2)
      oprot.writeI64(self.id)
      oprot.writeFieldEnd()
    if self.output_var != None:
      oprot.writeFieldBegin('output_var', TType.STRING, 3)
      oprot.writeString(self.output_var)
      oprot.writeFieldEnd()
    if self.description != None:
      oprot.writeFieldBegin('description', TType.LIST, 4)
      oprot.writeListBegin(TType.STRUCT, len(self.description))
      for iter32 in self.description:
        iter32.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.profiles != None:
      oprot.writeFieldBegin('profiles', TType.LIST, 5)
      oprot.writeListBegin(TType.STRUCT, len(self.profiles))
      for iter33 in self.profiles:
        iter33.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.branch_info != None:
      oprot.writeFieldBegin('branch_info', TType.STRUCT, 6)
      self.branch_info.write(oprot)
      oprot.writeFieldEnd()
    if self.dependencies != None:
      oprot.writeFieldBegin('dependencies', TType.LIST, 7)
      oprot.writeListBegin(TType.I64, len(self.dependencies))
      for iter34 in self.dependencies:
        oprot.writeI64(iter34)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.name is not None:
      value = pprint.pformat(self.name, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    name=%s' % (value))
    if self.id is not None:
      value = pprint.pformat(self.id, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    id=%s' % (value))
    if self.output_var is not None:
      value = pprint.pformat(self.output_var, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    output_var=%s' % (value))
    if self.description is not None:
      value = pprint.pformat(self.description, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    description=%s' % (value))
    if self.profiles is not None:
      value = pprint.pformat(self.profiles, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    profiles=%s' % (value))
    if self.branch_info is not None:
      value = pprint.pformat(self.branch_info, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    branch_info=%s' % (value))
    if self.dependencies is not None:
      value = pprint.pformat(self.dependencies, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    dependencies=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class PlanDescription:
  """
  Attributes:
   - plan_node_descs
   - node_index_map
   - format
   - optimize_time_in_us
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.LIST:
          self.plan_node_descs = []
          (_etype38, _size35) = iprot.readListBegin()
          if _size35 >= 0:
            for _i39 in six.moves.range(_size35):
              _elem40 = PlanNodeDescription()
              _elem40.read(iprot)
              self.plan_node_descs.append(_elem40)
          else: 
            while iprot.peekList():
              _elem41 = PlanNodeDescription()
              _elem41.read(iprot)
              self.plan_node_descs.append(_elem41)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.MAP:
          self.node_index_map = {}
          (_ktype43, _vtype44, _size42 ) = iprot.readMapBegin() 
          if _size42 >= 0:
            for _i46 in six.moves.range(_size42):
              _key47 = iprot.readI64()
              _val48 = iprot.readI64()
              self.node_index_map[_key47] = _val48
          else: 
            while iprot.peekMap():
              _key49 = iprot.readI64()
              _val50 = iprot.readI64()
              self.node_index_map[_key49] = _val50
          iprot.readMapEnd()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.format = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I32:
          self.optimize_time_in_us = iprot.readI32()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('PlanDescription')
    if self.plan_node_descs != None:
      oprot.writeFieldBegin('plan_node_descs', TType.LIST, 1)
      oprot.writeListBegin(TType.STRUCT, len(self.plan_node_descs))
      for iter51 in self.plan_node_descs:
        iter51.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    if self.node_index_map != None:
      oprot.writeFieldBegin('node_index_map', TType.MAP, 2)
      oprot.writeMapBegin(TType.I64, TType.I64, len(self.node_index_map))
      for kiter52,viter53 in self.node_index_map.items():
        oprot.writeI64(kiter52)
        oprot.writeI64(viter53)
      oprot.writeMapEnd()
      oprot.writeFieldEnd()
    if self.format != None:
      oprot.writeFieldBegin('format', TType.STRING, 3)
      oprot.writeString(self.format)
      oprot.writeFieldEnd()
    if self.optimize_time_in_us != None:
      oprot.writeFieldBegin('optimize_time_in_us', TType.I32, 4)
      oprot.writeI32(self.optimize_time_in_us)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.plan_node_descs is not None:
      value = pprint.pformat(self.plan_node_descs, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    plan_node_descs=%s' % (value))
    if self.node_index_map is not None:
      value = pprint.pformat(self.node_index_map, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    node_index_map=%s' % (value))
    if self.format is not None:
      value = pprint.pformat(self.format, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    format=%s' % (value))
    if self.optimize_time_in_us is not None:
      value = pprint.pformat(self.optimize_time_in_us, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    optimize_time_in_us=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class ExecutionResponse:
  """
  Attributes:
   - error_code
   - latency_in_us
   - data
   - space_name
   - error_msg
   - plan_desc
   - comment
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.error_code = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.latency_in_us = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.data = nebula2.common.ttypes.DataSet()
          self.data.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.space_name = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.error_msg = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRUCT:
          self.plan_desc = PlanDescription()
          self.plan_desc.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.STRING:
          self.comment = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('ExecutionResponse')
    if self.error_code != None:
      oprot.writeFieldBegin('error_code', TType.I32, 1)
      oprot.writeI32(self.error_code)
      oprot.writeFieldEnd()
    if self.latency_in_us != None:
      oprot.writeFieldBegin('latency_in_us', TType.I32, 2)
      oprot.writeI32(self.latency_in_us)
      oprot.writeFieldEnd()
    if self.data != None:
      oprot.writeFieldBegin('data', TType.STRUCT, 3)
      self.data.write(oprot)
      oprot.writeFieldEnd()
    if self.space_name != None:
      oprot.writeFieldBegin('space_name', TType.STRING, 4)
      oprot.writeString(self.space_name)
      oprot.writeFieldEnd()
    if self.error_msg != None:
      oprot.writeFieldBegin('error_msg', TType.STRING, 5)
      oprot.writeString(self.error_msg)
      oprot.writeFieldEnd()
    if self.plan_desc != None:
      oprot.writeFieldBegin('plan_desc', TType.STRUCT, 6)
      self.plan_desc.write(oprot)
      oprot.writeFieldEnd()
    if self.comment != None:
      oprot.writeFieldBegin('comment', TType.STRING, 7)
      oprot.writeString(self.comment)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.error_code is not None:
      value = pprint.pformat(self.error_code, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_code=%s' % (value))
    if self.latency_in_us is not None:
      value = pprint.pformat(self.latency_in_us, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    latency_in_us=%s' % (value))
    if self.data is not None:
      value = pprint.pformat(self.data, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    data=%s' % (value))
    if self.space_name is not None:
      value = pprint.pformat(self.space_name, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    space_name=%s' % (value))
    if self.error_msg is not None:
      value = pprint.pformat(self.error_msg, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_msg=%s' % (value))
    if self.plan_desc is not None:
      value = pprint.pformat(self.plan_desc, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    plan_desc=%s' % (value))
    if self.comment is not None:
      value = pprint.pformat(self.comment, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    comment=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class AuthResponse:
  """
  Attributes:
   - error_code
   - error_msg
   - session_id
   - time_zone_offset_seconds
   - time_zone_name
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.error_code = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.error_msg = iprot.readString()
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.session_id = iprot.readI64()
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I32:
          self.time_zone_offset_seconds = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.time_zone_name = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('AuthResponse')
    if self.error_code != None:
      oprot.writeFieldBegin('error_code', TType.I32, 1)
      oprot.writeI32(self.error_code)
      oprot.writeFieldEnd()
    if self.error_msg != None:
      oprot.writeFieldBegin('error_msg', TType.STRING, 2)
      oprot.writeString(self.error_msg)
      oprot.writeFieldEnd()
    if self.session_id != None:
      oprot.writeFieldBegin('session_id', TType.I64, 3)
      oprot.writeI64(self.session_id)
      oprot.writeFieldEnd()
    if self.time_zone_offset_seconds != None:
      oprot.writeFieldBegin('time_zone_offset_seconds', TType.I32, 4)
      oprot.writeI32(self.time_zone_offset_seconds)
      oprot.writeFieldEnd()
    if self.time_zone_name != None:
      oprot.writeFieldBegin('time_zone_name', TType.STRING, 5)
      oprot.writeString(self.time_zone_name)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.error_code is not None:
      value = pprint.pformat(self.error_code, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_code=%s' % (value))
    if self.error_msg is not None:
      value = pprint.pformat(self.error_msg, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_msg=%s' % (value))
    if self.session_id is not None:
      value = pprint.pformat(self.session_id, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    session_id=%s' % (value))
    if self.time_zone_offset_seconds is not None:
      value = pprint.pformat(self.time_zone_offset_seconds, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    time_zone_offset_seconds=%s' % (value))
    if self.time_zone_name is not None:
      value = pprint.pformat(self.time_zone_name, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    time_zone_name=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class VerifyClientVersionResp:
  """
  Attributes:
   - error_code
   - error_msg
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.error_code = iprot.readI32()
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.error_msg = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('VerifyClientVersionResp')
    if self.error_code != None:
      oprot.writeFieldBegin('error_code', TType.I32, 1)
      oprot.writeI32(self.error_code)
      oprot.writeFieldEnd()
    if self.error_msg != None:
      oprot.writeFieldBegin('error_msg', TType.STRING, 2)
      oprot.writeString(self.error_msg)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.error_code is not None:
      value = pprint.pformat(self.error_code, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_code=%s' % (value))
    if self.error_msg is not None:
      value = pprint.pformat(self.error_msg, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    error_msg=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

class VerifyClientVersionReq:
  """
  Attributes:
   - version
  """

  thrift_spec = None
  thrift_field_annotations = None
  thrift_struct_annotations = None
  __init__ = None
  @staticmethod
  def isUnion():
    return False

  def read(self, iprot):
    if (isinstance(iprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0)
      return
    if (isinstance(iprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(iprot, THeaderProtocol.THeaderProtocolAccelerate) and iprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastproto is not None:
      fastproto.decode(self, iprot.trans, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2)
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.version = iprot.readString()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if (isinstance(oprot, TBinaryProtocol.TBinaryProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_BINARY_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=0))
      return
    if (isinstance(oprot, TCompactProtocol.TCompactProtocolAccelerated) or (isinstance(oprot, THeaderProtocol.THeaderProtocolAccelerate) and oprot.get_protocol_id() == THeaderProtocol.THeaderProtocol.T_COMPACT_PROTOCOL)) and self.thrift_spec is not None and fastproto is not None:
      oprot.trans.write(fastproto.encode(self, [self.__class__, self.thrift_spec, False], utf8strings=UTF8STRINGS, protoid=2))
      return
    oprot.writeStructBegin('VerifyClientVersionReq')
    if self.version != None:
      oprot.writeFieldBegin('version', TType.STRING, 1)
      oprot.writeString(self.version)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def __repr__(self):
    L = []
    padding = ' ' * 4
    if self.version is not None:
      value = pprint.pformat(self.version, indent=0)
      value = padding.join(value.splitlines(True))
      L.append('    version=%s' % (value))
    return "%s(%s)" % (self.__class__.__name__, "\n" + ",\n".join(L) if L else '')

  def __eq__(self, other):
    if not isinstance(other, self.__class__):
      return False

    return self.__dict__ == other.__dict__ 

  def __ne__(self, other):
    return not (self == other)

  # Override the __hash__ function for Python3 - t10434117
  if not six.PY2:
    __hash__ = object.__hash__

all_structs.append(ProfilingStats)
ProfilingStats.thrift_spec = (
  None, # 0
  (1, TType.I64, 'rows', None, None, 0, ), # 1
  (2, TType.I64, 'exec_duration_in_us', None, None, 0, ), # 2
  (3, TType.I64, 'total_duration_in_us', None, None, 0, ), # 3
  (4, TType.MAP, 'other_stats', (TType.STRING,False,TType.STRING,False), None, 1, ), # 4
)

ProfilingStats.thrift_struct_annotations = {
  "cpp.type": "nebula::ProfilingStats",
}
ProfilingStats.thrift_field_annotations = {
}

def ProfilingStats__init__(self, rows=None, exec_duration_in_us=None, total_duration_in_us=None, other_stats=None,):
  self.rows = rows
  self.exec_duration_in_us = exec_duration_in_us
  self.total_duration_in_us = total_duration_in_us
  self.other_stats = other_stats

ProfilingStats.__init__ = ProfilingStats__init__

def ProfilingStats__setstate__(self, state):
  state.setdefault('rows', None)
  state.setdefault('exec_duration_in_us', None)
  state.setdefault('total_duration_in_us', None)
  state.setdefault('other_stats', None)
  self.__dict__ = state

ProfilingStats.__getstate__ = lambda self: self.__dict__.copy()
ProfilingStats.__setstate__ = ProfilingStats__setstate__

all_structs.append(PlanNodeBranchInfo)
PlanNodeBranchInfo.thrift_spec = (
  None, # 0
  (1, TType.BOOL, 'is_do_branch', None, None, 0, ), # 1
  (2, TType.I64, 'condition_node_id', None, None, 0, ), # 2
)

PlanNodeBranchInfo.thrift_struct_annotations = {
  "cpp.type": "nebula::PlanNodeBranchInfo",
}
PlanNodeBranchInfo.thrift_field_annotations = {
}

def PlanNodeBranchInfo__init__(self, is_do_branch=None, condition_node_id=None,):
  self.is_do_branch = is_do_branch
  self.condition_node_id = condition_node_id

PlanNodeBranchInfo.__init__ = PlanNodeBranchInfo__init__

def PlanNodeBranchInfo__setstate__(self, state):
  state.setdefault('is_do_branch', None)
  state.setdefault('condition_node_id', None)
  self.__dict__ = state

PlanNodeBranchInfo.__getstate__ = lambda self: self.__dict__.copy()
PlanNodeBranchInfo.__setstate__ = PlanNodeBranchInfo__setstate__

all_structs.append(Pair)
Pair.thrift_spec = (
  None, # 0
  (1, TType.STRING, 'key', False, None, 0, ), # 1
  (2, TType.STRING, 'value', False, None, 0, ), # 2
)

Pair.thrift_struct_annotations = {
  "cpp.type": "nebula::Pair",
}
Pair.thrift_field_annotations = {
}

def Pair__init__(self, key=None, value=None,):
  self.key = key
  self.value = value

Pair.__init__ = Pair__init__

def Pair__setstate__(self, state):
  state.setdefault('key', None)
  state.setdefault('value', None)
  self.__dict__ = state

Pair.__getstate__ = lambda self: self.__dict__.copy()
Pair.__setstate__ = Pair__setstate__

all_structs.append(PlanNodeDescription)
PlanNodeDescription.thrift_spec = (
  None, # 0
  (1, TType.STRING, 'name', False, None, 0, ), # 1
  (2, TType.I64, 'id', None, None, 0, ), # 2
  (3, TType.STRING, 'output_var', False, None, 0, ), # 3
  (4, TType.LIST, 'description', (TType.STRUCT,[Pair, Pair.thrift_spec, False]), None, 1, ), # 4
  (5, TType.LIST, 'profiles', (TType.STRUCT,[ProfilingStats, ProfilingStats.thrift_spec, False]), None, 1, ), # 5
  (6, TType.STRUCT, 'branch_info', [PlanNodeBranchInfo, PlanNodeBranchInfo.thrift_spec, False], None, 1, ), # 6
  (7, TType.LIST, 'dependencies', (TType.I64,None), None, 1, ), # 7
)

PlanNodeDescription.thrift_struct_annotations = {
  "cpp.type": "nebula::PlanNodeDescription",
}
PlanNodeDescription.thrift_field_annotations = {
}

def PlanNodeDescription__init__(self, name=None, id=None, output_var=None, description=None, profiles=None, branch_info=None, dependencies=None,):
  self.name = name
  self.id = id
  self.output_var = output_var
  self.description = description
  self.profiles = profiles
  self.branch_info = branch_info
  self.dependencies = dependencies

PlanNodeDescription.__init__ = PlanNodeDescription__init__

def PlanNodeDescription__setstate__(self, state):
  state.setdefault('name', None)
  state.setdefault('id', None)
  state.setdefault('output_var', None)
  state.setdefault('description', None)
  state.setdefault('profiles', None)
  state.setdefault('branch_info', None)
  state.setdefault('dependencies', None)
  self.__dict__ = state

PlanNodeDescription.__getstate__ = lambda self: self.__dict__.copy()
PlanNodeDescription.__setstate__ = PlanNodeDescription__setstate__

all_structs.append(PlanDescription)
PlanDescription.thrift_spec = (
  None, # 0
  (1, TType.LIST, 'plan_node_descs', (TType.STRUCT,[PlanNodeDescription, PlanNodeDescription.thrift_spec, False]), None, 0, ), # 1
  (2, TType.MAP, 'node_index_map', (TType.I64,None,TType.I64,None), None, 0, ), # 2
  (3, TType.STRING, 'format', False, None, 0, ), # 3
  (4, TType.I32, 'optimize_time_in_us', None, None, 0, ), # 4
)

PlanDescription.thrift_struct_annotations = {
  "cpp.type": "nebula::PlanDescription",
}
PlanDescription.thrift_field_annotations = {
}

def PlanDescription__init__(self, plan_node_descs=None, node_index_map=None, format=None, optimize_time_in_us=None,):
  self.plan_node_descs = plan_node_descs
  self.node_index_map = node_index_map
  self.format = format
  self.optimize_time_in_us = optimize_time_in_us

PlanDescription.__init__ = PlanDescription__init__

def PlanDescription__setstate__(self, state):
  state.setdefault('plan_node_descs', None)
  state.setdefault('node_index_map', None)
  state.setdefault('format', None)
  state.setdefault('optimize_time_in_us', None)
  self.__dict__ = state

PlanDescription.__getstate__ = lambda self: self.__dict__.copy()
PlanDescription.__setstate__ = PlanDescription__setstate__

all_structs.append(ExecutionResponse)
ExecutionResponse.thrift_spec = (
  None, # 0
  (1, TType.I32, 'error_code', nebula2.common.ttypes.ErrorCode, None, 0, ), # 1
  (2, TType.I32, 'latency_in_us', None, None, 0, ), # 2
  (3, TType.STRUCT, 'data', [nebula2.common.ttypes.DataSet, nebula2.common.ttypes.DataSet.thrift_spec, False], None, 1, ), # 3
  (4, TType.STRING, 'space_name', False, None, 1, ), # 4
  (5, TType.STRING, 'error_msg', False, None, 1, ), # 5
  (6, TType.STRUCT, 'plan_desc', [PlanDescription, PlanDescription.thrift_spec, False], None, 1, ), # 6
  (7, TType.STRING, 'comment', False, None, 1, ), # 7
)

ExecutionResponse.thrift_struct_annotations = {
  "cpp.type": "nebula::ExecutionResponse",
}
ExecutionResponse.thrift_field_annotations = {
}

def ExecutionResponse__init__(self, error_code=None, latency_in_us=None, data=None, space_name=None, error_msg=None, plan_desc=None, comment=None,):
  self.error_code = error_code
  self.latency_in_us = latency_in_us
  self.data = data
  self.space_name = space_name
  self.error_msg = error_msg
  self.plan_desc = plan_desc
  self.comment = comment

ExecutionResponse.__init__ = ExecutionResponse__init__

def ExecutionResponse__setstate__(self, state):
  state.setdefault('error_code', None)
  state.setdefault('latency_in_us', None)
  state.setdefault('data', None)
  state.setdefault('space_name', None)
  state.setdefault('error_msg', None)
  state.setdefault('plan_desc', None)
  state.setdefault('comment', None)
  self.__dict__ = state

ExecutionResponse.__getstate__ = lambda self: self.__dict__.copy()
ExecutionResponse.__setstate__ = ExecutionResponse__setstate__

all_structs.append(AuthResponse)
AuthResponse.thrift_spec = (
  None, # 0
  (1, TType.I32, 'error_code', nebula2.common.ttypes.ErrorCode, None, 0, ), # 1
  (2, TType.STRING, 'error_msg', False, None, 1, ), # 2
  (3, TType.I64, 'session_id', None, None, 1, ), # 3
  (4, TType.I32, 'time_zone_offset_seconds', None, None, 1, ), # 4
  (5, TType.STRING, 'time_zone_name', False, None, 1, ), # 5
)

AuthResponse.thrift_struct_annotations = {
  "cpp.type": "nebula::AuthResponse",
}
AuthResponse.thrift_field_annotations = {
}

def AuthResponse__init__(self, error_code=None, error_msg=None, session_id=None, time_zone_offset_seconds=None, time_zone_name=None,):
  self.error_code = error_code
  self.error_msg = error_msg
  self.session_id = session_id
  self.time_zone_offset_seconds = time_zone_offset_seconds
  self.time_zone_name = time_zone_name

AuthResponse.__init__ = AuthResponse__init__

def AuthResponse__setstate__(self, state):
  state.setdefault('error_code', None)
  state.setdefault('error_msg', None)
  state.setdefault('session_id', None)
  state.setdefault('time_zone_offset_seconds', None)
  state.setdefault('time_zone_name', None)
  self.__dict__ = state

AuthResponse.__getstate__ = lambda self: self.__dict__.copy()
AuthResponse.__setstate__ = AuthResponse__setstate__

all_structs.append(VerifyClientVersionResp)
VerifyClientVersionResp.thrift_spec = (
  None, # 0
  (1, TType.I32, 'error_code', nebula2.common.ttypes.ErrorCode, None, 0, ), # 1
  (2, TType.STRING, 'error_msg', False, None, 1, ), # 2
)

VerifyClientVersionResp.thrift_struct_annotations = {
}
VerifyClientVersionResp.thrift_field_annotations = {
}

def VerifyClientVersionResp__init__(self, error_code=None, error_msg=None,):
  self.error_code = error_code
  self.error_msg = error_msg

VerifyClientVersionResp.__init__ = VerifyClientVersionResp__init__

def VerifyClientVersionResp__setstate__(self, state):
  state.setdefault('error_code', None)
  state.setdefault('error_msg', None)
  self.__dict__ = state

VerifyClientVersionResp.__getstate__ = lambda self: self.__dict__.copy()
VerifyClientVersionResp.__setstate__ = VerifyClientVersionResp__setstate__

all_structs.append(VerifyClientVersionReq)
VerifyClientVersionReq.thrift_spec = (
  None, # 0
  (1, TType.STRING, 'version', False, "2.6.0", 0, ), # 1
)

VerifyClientVersionReq.thrift_struct_annotations = {
}
VerifyClientVersionReq.thrift_field_annotations = {
}

def VerifyClientVersionReq__init__(self, version=VerifyClientVersionReq.thrift_spec[1][4],):
  self.version = version

VerifyClientVersionReq.__init__ = VerifyClientVersionReq__init__

def VerifyClientVersionReq__setstate__(self, state):
  state.setdefault('version', "2.6.0")
  self.__dict__ = state

VerifyClientVersionReq.__getstate__ = lambda self: self.__dict__.copy()
VerifyClientVersionReq.__setstate__ = VerifyClientVersionReq__setstate__

fix_spec(all_structs)
del all_structs



#!/usr/bin/env python
# --coding:utf-8--

# Copyright (c) 2020 vesoft inc. All rights reserved.
#
# This source code is licensed under Apache 2.0 License,
# attached with Common Clause Condition 1.0, found in the LICENSES directory.

"""
The client to scan vertex and edge from storage,
the return data is from thr graph database
"""
import logging
import sys

from nebula2.sclient.ScanResult import ScanResult
from nebula2.sclient.net import GraphStorageConnection
from nebula2.storage.ttypes import (
    ScanEdgeRequest,
    ScanVertexRequest,
    VertexProp,
    EdgeProp
)

kVid = b'_vid'
kSrc = b'_src'
kType = b'_type'
kRank = b'_rank'
kDst = b'_dst'


class GraphStorageClient(object):
    DEFAULT_START_TIME = 0
    DEFAULT_END_TIME = sys.maxsize
    DEFAULT_LIMIT = 1000

    def __init__(self, meta_cache, storage_addrs=None, time_out=60000):
        self._meta_cache = meta_cache
        self._storage_addrs = storage_addrs
        self._time_out = time_out
        self._connections = []
        self._create_connection()

    def get_conns(self):
        """get all connections which connect to storaged, the ScanResult use it

        :return: list<GraphStorageConnection>
        """
        return self._connections

    def __del__(self):
        self.close()

    def close(self):
        """close the GraphStorageClient

        :return:
        """
        try:
            for conn in self._connections:
                conn.close()
        except Exception as e:
            logging.error('Close connection failed: {}'.format(e))
            raise

    def _create_connection(self):
        """create GraphStorageConnection

        :return: GraphStorageConnection
        """
        if self._storage_addrs is None:
            self._storage_addrs = self._meta_cache.get_all_storage_addrs()
        if len(self._storage_addrs) == 0:
            raise RuntimeError('Get storage address from meta cache is empty')
        try:
            for addr in self._storage_addrs:
                conn = GraphStorageConnection(addr, self._time_out, self._meta_cache)
                conn.open()
                self._connections.append(conn)
        except Exception as e:
            logging.error('Create storage connection failed: {}'.format(e))
            raise

    def get_space_addrs(self, space_name):
        """get all storage addresses that manage space

        :param space_name: the specified space name
        :return: list<(ip, port)>
        """
        return self.meta_cache.get_space_addrs(space_name)

    def scan_vertex(self,
                    space_name,
                    tag_name,
                    prop_names=[],
                    limit=DEFAULT_LIMIT,
                    start_time=DEFAULT_START_TIME,
                    end_time=DEFAULT_END_TIME,
                    where=None,
                    only_latest_version=False,
                    enable_read_from_follower=True,
                    partial_success=False):
        """scan vertex with the specified space_name, tag_name,
        if the prop_names is empty, will return all properties of the tag

        :param prop_names: if given empty, return all property
        :param tag_name: the tag name
        :param space_name: the space name
        :param limit: the max vertex number from one storaged
        :param start_time: the min version of vertex
        :param end_time: the max version of vertex
        :param where: now is unsupported
        :param only_latest_version: when storage enable multi versions and only_latest_version is true,
        only return latest version.
        when storage disable multi versions, just use the default value.
        :param enable_read_from_follower: if set to false, forbid follower read
        :param partial_success: if set true, when partial success, it will continue until finish
        :return: ScanResult
        """
        part_leaders = self._meta_cache.get_part_leaders(space_name)
        return self._scan_vertex(space_name,
                                 part_leaders,
                                 tag_name,
                                 prop_names,
                                 limit,
                                 start_time,
                                 end_time,
                                 where,
                                 only_latest_version,
                                 enable_read_from_follower,
                                 partial_success)

    def scan_vertex_with_part(self,
                              space_name,
                              part,
                              tag_name,
                              prop_names=[],
                              limit=DEFAULT_LIMIT,
                              start_time=DEFAULT_START_TIME,
                              end_time=DEFAULT_END_TIME,
                              where=None,
                              only_latest_version=False,
                              enable_read_from_follower=True,
                              partial_success=False):
        """scan vertex with the specified space_name, partId, tag_name,
        if the prop_names is empty, will return all properties of the tag

        :param prop_names: if given empty, return all property
        :param tag_name: the tag name
        :type part: part id
        :param space_name: the space name
        :param limit: the max vertex number from one storaged
        :param start_time: the min version of vertex
        :param end_time: the max version of vertex
        :param where: now is unsupported
        :param only_latest_version: when storage enable multi versions and only_latest_version is true,
        only return latest version.
        when storage disable multi versions, just use the default value.
        :param enable_read_from_follower: if set to false, forbid follower read
        :param partial_success: if set true, when partial success, it will continue until finish
        :return: ScanResult
        """

        part_leaders = {part: self._meta_cache.get_part_leader(space_name, part)}
        return self._scan_vertex(space_name,
                                 part_leaders,
                                 tag_name,
                                 prop_names,
                                 limit,
                                 start_time,
                                 end_time,
                                 where,
                                 only_latest_version,
                                 enable_read_from_follower,
                                 partial_success)

    def _scan_vertex(self,
                     space_name,
                     part_leaders,
                     tag_name,
                     prop_names,
                     limit,
                     start_time,
                     end_time,
                     where,
                     only_latest_version,
                     enable_read_from_follower,
                     partial_success=False):
        space_id = self._meta_cache.get_space_id(space_name)
        tag_id = self._meta_cache.get_tag_id(space_name, tag_name)
        vertex_prop: VertexProp = VertexProp()
        vertex_prop.tag = tag_id
        vertex_prop.props = [kVid]
        for prop_name in prop_names:
            vertex_prop.props.append(prop_name.encode('utf-8'))

        if len(prop_names) == 0:
            schema = self._meta_cache.get_tag_schema(space_name, tag_name)
            for col in schema.columns:
                vertex_prop.props.append(col.name)

        req = ScanVertexRequest()
        req.space_id = space_id
        req.part_id = 0
        req.return_columns = vertex_prop
        req.limit = limit
        req.start_time = start_time
        req.end_time = end_time
        req.filter = where
        req.only_latest_version = only_latest_version
        req.enable_read_from_follower = enable_read_from_follower
        return ScanResult(self,
                          req=req,
                          part_addrs=part_leaders,
                          is_vertex=True,
                          partial_success=partial_success)

    def scan_edge(self,
                  space_name,
                  edge_name,
                  prop_names=[],
                  limit=DEFAULT_LIMIT,
                  start_time=DEFAULT_START_TIME,
                  end_time=DEFAULT_END_TIME,
                  where=None,
                  only_latest_version=False,
                  enable_read_from_follower=True,
                  partial_success=False):
        """scan edge with the specified space_name, edge_name,
        if the prop_names is empty, will return all properties of the edge

        :param prop_names: if given empty, return all property
        :param edge_name: the edge name
        :param space_name: the space name
        :param limit: the max vertex number from one storaged
        :param start_time: the min version of vertex
        :param end_time: the max version of vertex
        :param where: now is unsupported
        :param only_latest_version: when storage enable multi versions and only_latest_version is true,
        only return latest version.
        when storage disable multi versions, just use the default value.
        :param enable_read_from_follower: if set to false, forbid follower read
        :param partial_success: if set true, when partial success, it will continue until finish
        :return: ScanResult
        """
        part_leaders = self._meta_cache.get_part_leaders(space_name)
        return self._scan_edge(space_name,
                               part_leaders,
                               edge_name,
                               prop_names,
                               limit,
                               start_time,
                               end_time,
                               where,
                               only_latest_version,
                               enable_read_from_follower,
                               partial_success)

    def scan_edge_with_part(self,
                            space_name,
                            part,
                            edge_name,
                            prop_names=[],
                            limit=DEFAULT_LIMIT,
                            start_time=DEFAULT_START_TIME,
                            end_time=DEFAULT_END_TIME,
                            where=None,
                            only_latest_version=False,
                            enable_read_from_follower=True,
                            partial_success=False):
        """scan edge with the specified space_name, partId, edge_name,
        if the prop_names is empty, will return all properties of the edge

        :param space_name: the space name
        :param part: the partition num of the given space
        :type prop_names: if given empty, return all property
        :param edge_name: the edge name
        :param limit: the max vertex number from one storaged
        :param start_time: the min version of edge
        :param end_time: the max version of edge
        :param where: now is unsupported
        :param only_latest_version: when storage enable multi versions and only_latest_version is true,
        only return latest version.
        when storage disable multi versions, just use the default value.
        :param enable_read_from_follower: if set to false, forbid follower read
        :param partial_success: if set true, when partial success, it will continue until finish
        :return: ScanResult
        """
        part_leaders = {part: self._meta_cache.get_part_leader(space_name, part)}
        return self._scan_edge(space_name,
                               part_leaders,
                               edge_name,
                               prop_names,
                               limit,
                               start_time,
                               end_time,
                               where,
                               only_latest_version,
                               enable_read_from_follower,
                               partial_success)

    def _scan_edge(self,
                   space_name,
                   part_leaders,
                   edge_name,
                   prop_names,
                   limit,
                   start_time,
                   end_time,
                   where,
                   only_latest_version,
                   enable_read_from_follower,
                   partial_success):
        space_id = self._meta_cache.get_space_id(space_name)
        edge_type = self._meta_cache.get_edge_type(space_name, edge_name)
        edge_prop = EdgeProp()
        edge_prop.type = edge_type
        edge_prop.props = [kSrc, kType, kRank, kDst]

        for prop_name in prop_names:
            edge_prop.props.append(prop_name.encode('utf-8'))

        # When storage return column names, here need delete
        if len(prop_names) == 0:
            schema = self._meta_cache.get_edge_schema(space_name, edge_name)
            for col in schema.columns:
                edge_prop.props.append(col.name)

        req = ScanEdgeRequest()
        req.space_id = space_id
        req.part_id = 0
        req.return_columns = edge_prop
        req.limit = limit
        req.start_time = start_time
        req.end_time = end_time
        req.filter = where
        req.only_latest_version = only_latest_version
        req.enable_read_from_follower = enable_read_from_follower
        return ScanResult(self,
                          req=req,
                          part_addrs=part_leaders,
                          is_vertex=False,
                          partial_success=partial_success)




# Copyright (c) Facebook, Inc. and its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pyre-unsafe

"""
Support classes for generating random values for thrift types
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from nebula2.fbthrift import Thrift


def get_spec(ttype, spec_args):
    """Create or get an existing introspection wrapper for a thrift type.

    Args:
        ttype (Thrift.TType): the thrift type
        spec_args: optional type-dependent additional information generated by
        the thrift compiler (e.g. the type of a list's elements)
    """
    for s in get_spec.cache:
        if s.ttype == ttype and s.spec_args == spec_args:
            return s

    # enums and i32s share the same thrift type. They can be distinguished
    # based on spec_args, which is None for ints
    if ttype == Thrift.TType.I32 and spec_args:
        return ThriftEnumSpec(ttype, spec_args)
    type_map = {
        Thrift.TType.BOOL: ThriftBoolSpec,
        Thrift.TType.BYTE: ThriftByteSpec,
        Thrift.TType.I16: ThriftI16Spec,
        Thrift.TType.I32: ThriftI32Spec,
        Thrift.TType.I64: ThriftI64Spec,
        Thrift.TType.FLOAT: ThriftFloatSpec,
        Thrift.TType.DOUBLE: ThriftDoubleSpec,
        Thrift.TType.STRING: ThriftScalarSpec,
        Thrift.TType.LIST: ThriftListSpec,
        Thrift.TType.SET: ThriftSetSpec,
        Thrift.TType.MAP: ThriftMapSpec,
        Thrift.TType.STRUCT: ThriftStructSpec,
    }
    spec = type_map[ttype](ttype, spec_args)
    get_spec.cache.append(spec)
    spec._init_subelements()
    return spec


get_spec.cache = []


class ThriftTypeSpec(object):
    """Base class that abstracts the functionality required by the randomizer"""
    SUBTYPE_ELEMENT = "element"
    SUBTYPE_KEY = "key"
    SUBTYPE_VALUE = "value"

    def __init__(self, ttype):
        self.ttype = ttype

    @classmethod
    def is_enum(cls):
        return False

    def get_type_name(self):
        raise NotImplementedError

    def get_type(self):
        raise NotImplementedError

    def construct_instance(self, *args, **kwargs):
        """Construct a thrift type instance from appropriate arguments"""
        return self.get_type()(*args, **kwargs)

    def get_subtypes(self):
        """Return a mapping from each subtype to its ThriftTypeSpec. Non-empty
        for lists, sets, maps and structs. The key identifies the field name
        for structs; for other types, names defined above are used
        """
        return {}

    def get_subtype_requiredness(self):
        """Return a mapping from each subtype to its requiredness. The keys are
        the same as for get_subtypes and the values are booleans
        """
        return {}

    def value_to_dict(self, thrift_value):
        """Return a mapping from field names to values. Only required for
        structs
        """
        raise NotImplementedError


class ThriftPyTypeSpec(ThriftTypeSpec):
    """TypeSpec implementation for python thrift structures"""
    name = ""

    def __init__(self, ttype, spec_args):
        super(ThriftPyTypeSpec, self).__init__(ttype)
        self.spec_args = spec_args

    def __eq__(self, other):
        return (
            self.ttype == other.ttype
            and self.spec_args == other.spec_args
        )

    def _init_subelements(self):
        pass


class ThriftScalarSpec(ThriftPyTypeSpec):
    def get_type_name(self):
        return self.name


class ThriftEnumSpec(ThriftPyTypeSpec):
    @classmethod
    def is_enum(cls):
        return True

    def names_to_values(self):
        return self.spec_args._NAMES_TO_VALUES

    def get_type_name(self):
        return self.spec_args.__name__

    def get_type(self):
        return self.spec_args

    def construct_instance(self, value):
        # enums are ints. The thrift enum type (self.spec_args) is not supposed to be
        # instantiated
        return int(value)


def _build_scalar_type_spec(_name):
    class _ScalarTypeSpec(ThriftScalarSpec):
        name = _name

    return _ScalarTypeSpec


ThriftBoolSpec = _build_scalar_type_spec("bool")
ThriftByteSpec = _build_scalar_type_spec("byte")
ThriftI16Spec = _build_scalar_type_spec("i16")
ThriftI32Spec = _build_scalar_type_spec("i32")
ThriftI64Spec = _build_scalar_type_spec("i64")
ThriftFloatSpec = _build_scalar_type_spec("float")
ThriftDoubleSpec = _build_scalar_type_spec("double")
ThriftStringSpec = _build_scalar_type_spec("string")


class ThriftNonAssociativeContainerSpec(ThriftPyTypeSpec):
    def _init_subelements(self):
        elem_ttype, elem_spec_args = self.spec_args
        self.elem_spec = get_spec(elem_ttype, elem_spec_args)

    def get_type_name(self):
        return "{}<{}>".format(self.name, self.elem_spec.get_type_name())

    def get_subtypes(self):
        return {self.SUBTYPE_ELEMENT: self.elem_spec}


class ThriftListSpec(ThriftNonAssociativeContainerSpec):
    name = "list"

    def get_type(self):
        return list

class ThriftSetSpec(ThriftNonAssociativeContainerSpec):
    name = "set"

    def get_type(self):
        return set

class ThriftMapSpec(ThriftPyTypeSpec):
    name = "map"

    def get_type(self):
        return dict

    def _init_subelements(self):
        key_ttype, key_spec_args, val_ttype, val_spec_args = self.spec_args
        self.key_spec = get_spec(key_ttype, key_spec_args)
        self.val_spec = get_spec(val_ttype, val_spec_args)

    def get_type_name(self):
        return "{}<{}, {}>".format(
            self.name,
            self.key_spec.get_type_name(),
            self.val_spec.get_type_name(),
        )

    def get_subtypes(self):
        return {
            self.SUBTYPE_KEY: self.key_spec,
            self.SUBTYPE_VALUE: self.val_spec
        }


class ThriftStructSpec(ThriftPyTypeSpec):
    def _is_required(self, required_value):
        """Enum defined in /thrift/compiler/parse/t_field.h:

        T_REQUIRED = 0
        T_OPTIONAL = 1
        T_OPT_IN_REQ_OUT = 2

        Return True iff required_value is T_REQUIRED
        """
        return required_value == 0

    def _init_subelements(self):
        # For a TType.STRUCT, the spec_args is a triple
        # (py thrift type, py thrift type spec, kind),
        # where kind is True for a union and False otherwise
        self.type_class, self.type_spec, self.is_union = self.spec_args

        self.subtypes = {}
        self.subtype_requiredness = {}
        for attr_spec in self.type_spec:
            if attr_spec is None:
                continue
            _, ttype, attr_name, spec_args, _, req = attr_spec
            self.subtypes[attr_name] = get_spec(ttype, spec_args)
            self.subtype_requiredness[attr_name] = self._is_required(req)

    def get_type_name(self):
        return self.type_class.__name__

    def get_type(self):
        return self.type_class

    def get_subtypes(self):
        return self.subtypes

    def get_subtype_requiredness(self):
        return self.subtype_requiredness

    def value_to_dict(self, thrift_value):
        if isinstance(thrift_value, dict):
            return thrift_value
        res = {}
        if self.is_union:
            field = getattr(thrift_value, 'field', None)
            value = getattr(thrift_value, 'value', None)
            thrift_spec = getattr(thrift_value, 'thrift_spec', None)
            if not field or not thrift_spec:
                return res
            field_spec = thrift_spec[field]
            if not field_spec or len(field_spec) < 3:
                return res
            res[field_spec[2]] = value
        else:
            for key, value in vars(thrift_value).items():
                if value is not None:
                    res[key] = value
        return res



# Copyright (c) Facebook, Inc. and its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pyre-unsafe

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import sys
import os
import threading
if sys.version_info[0] >= 3:
    import queue
    # pyre-fixme[11]: Annotation `queue` is not defined as a type.
    Queue = queue
else:
    import Queue
import warnings

from nebula2.fbthrift.Thrift import TProcessor, TApplicationException
from nebula2.fbthrift.transport import TTransport
from nebula2.fbthrift.protocol import TBinaryProtocol
from nebula2.fbthrift.protocol.THeaderProtocol import THeaderProtocolFactory

class TConnectionContext:
    def getPeerName(self):
        """Gets the address of the client.

        Returns:
          The equivalent value of socket.getpeername() on the client socket
        """
        raise NotImplementedError


class TRpcConnectionContext(TConnectionContext):
    """Connection context class for thrift RPC calls"""
    def __init__(self, client_socket, iprot=None, oprot=None):
        """Initializer.

        Arguments:
          client_socket: the TSocket to the client
        """
        self._client_socket = client_socket
        self.iprot = iprot
        self.oprot = oprot

    def setProtocols(self, iprot, oprot):
        self.iprot = iprot
        self.oprot = oprot

    def getPeerName(self):
        """Gets the address of the client.

        Returns:
          Same value as socket.peername() for the TSocket
        """
        return self._client_socket.getPeerName()

    def getSockName(self):
        """Gets the address of the server.

        Returns:
          Same value as socket.getsockname() for the TSocket
        """
        return self._client_socket.getsockname()


class TServerEventHandler:
    """Event handler base class.

    Override selected methods on this class to implement custom event handling
    """
    def preServe(self, address):
        """Called before the server begins.

        Arguments:
          address: the address that the server is listening on
        """
        pass

    def newConnection(self, context):
        """Called when a client has connected and is about to begin processing.

        Arguments:
          context: instance of TRpcConnectionContext
        """
        pass

    def clientBegin(self, iprot, oprot):
        """Deprecated: Called when a new connection is made to the server.

        For all servers other than TNonblockingServer, this function is called
        whenever newConnection is called and vice versa.  This is the old-style
        for event handling and is not supported for TNonblockingServer. New
        code should always use the newConnection method.
        """
        pass

    def connectionDestroyed(self, context):
        """Called when a client has finished request-handling.

        Arguments:
          context: instance of TRpcConnectionContext
        """
        pass


class TServer:

    """Base interface for a server, which must have a serve method."""

    """ constructors for all servers:
    1) (processor, serverTransport)
    2) (processor, serverTransport, transportFactory, protocolFactory)
    3) (processor, serverTransport,
        inputTransportFactory, outputTransportFactory,
        inputProtocolFactory, outputProtocolFactory)

        Optionally, the handler can be passed instead of the processor,
        and a processor will be created automatically:

    4) (handler, serverTransport)
    5) (handler, serverTransport, transportFacotry, protocolFactory)
    6) (handler, serverTransport,
        inputTransportFactory, outputTransportFactory,
        inputProtocolFactory, outputProtocolFactory)

        The attribute serverEventHandler (default: None) receives
        callbacks for various events in the server lifecycle.  It should
        be set to an instance of TServerEventHandler.

        """
    def __init__(self, *args):
        if (len(args) == 2):
            self.__initArgs__(args[0], args[1],
                              TTransport.TTransportFactoryBase(),
                              TTransport.TTransportFactoryBase(),
                              TBinaryProtocol.TBinaryProtocolFactory(),
                              TBinaryProtocol.TBinaryProtocolFactory())
        elif (len(args) == 4):
            self.__initArgs__(args[0], args[1], args[2], args[2], args[3],
                    args[3])
        elif (len(args) == 6):
            self.__initArgs__(args[0], args[1], args[2], args[3], args[4],
                    args[5])

    def __initArgs__(self, processor, serverTransport,
                     inputTransportFactory, outputTransportFactory,
                     inputProtocolFactory, outputProtocolFactory):
        self.processor = self._getProcessor(processor)
        self.serverTransport = serverTransport
        self.inputTransportFactory = inputTransportFactory
        self.outputTransportFactory = outputTransportFactory
        self.inputProtocolFactory = inputProtocolFactory
        self.outputProtocolFactory = outputProtocolFactory

        self.serverEventHandler = TServerEventHandler()

    def _getProcessor(self, processor):
        """ Check if a processor is really a processor, or if it is a handler
            auto create a processor for it """
        if isinstance(processor, TProcessor):
            return processor
        elif hasattr(processor, "_processor_type"):
            handler = processor
            return handler._processor_type(handler)
        else:
            raise TApplicationException(
                    message="Could not detect processor type")

    def setServerEventHandler(self, handler):
        self.serverEventHandler = handler

    def _clientBegin(self, context, iprot, oprot):
        self.serverEventHandler.newConnection(context)
        self.serverEventHandler.clientBegin(iprot, oprot)

    def handle(self, client):
        itrans = self.inputTransportFactory.getTransport(client)
        otrans = self.outputTransportFactory.getTransport(client)
        iprot = self.inputProtocolFactory.getProtocol(itrans)

        if isinstance(self.inputProtocolFactory, THeaderProtocolFactory):
            oprot = iprot
        else:
            oprot = self.outputProtocolFactory.getProtocol(otrans)

        context = TRpcConnectionContext(client, iprot, oprot)
        self._clientBegin(context, iprot, oprot)

        try:
            while True:
                self.processor.process(iprot, oprot, context)
        except TTransport.TTransportException:
            pass
        except Exception as x:
            logging.exception(x)

        self.serverEventHandler.connectionDestroyed(context)
        itrans.close()
        otrans.close()

    def serve(self):
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio

from nebula2.fbthrift.server.TAsyncioServer import ThriftClientProtocolFactory
from nebula2.fbthrift.util.Decorators import protocol_manager


class async_protocol_manager:
    def __init__(self, coro):
        """
        Given a coro from create_connection create a context manager
        around the protocol returned
        """
        self.coro = coro

    def __await__(self):
        async def as_protocol_manager():
            _, protocol = await self.coro
            return protocol_manager(protocol)

        return as_protocol_manager().__await__()

    __iter__ = __await__

    async def __aenter__(self):
        _, self.protocol = await self.coro
        return self.protocol.client

    async def __aexit__(self, exc_type, exc, tb):
        self.protocol.close()


def create_client(
        client_klass,
        *,
        host=None,
        port=None,
        sock=None,
        loop=None,
        timeouts=None,
        client_type=None,
        ssl=None,
):
    """
    create an asyncio thrift client and return an async context
    manager that can be used as follows:

    async with create_client(smc2_client, port=1421) as smc:
        await smc.getStatus()

    This can be used in the old way:

    with (await create_client(smc2_client, port=1421)) as smc:
        await smc.getStatus()

    or even the old deprecated way:

    with (yield from create_client(smc2_client, port=1421) as smc:
        yield from smc.getStatus()

    :param client_klass: thrift Client class
    :param host: hostname/ip, None = loopback
    :param port: port number
    :param sock: socket.socket object
    :param loop: asyncio event loop
    :returns: an Async Context Manager
    """
    if not loop:
        loop = asyncio.get_event_loop()

    coro = loop.create_connection(
        ThriftClientProtocolFactory(
            client_klass,
            loop=loop,
            timeouts=timeouts,
            client_type=client_type,
        ),
        host=host,
        port=port,
        sock=sock,
        ssl=ssl,
    )
    return async_protocol_manager(coro)


def call_as_future(f, loop, *args, **kwargs):
    """call_as_future(callable, *args, **kwargs) -> asyncio.Task

    Like asyncio.ensure_future() but takes any callable and converts
    it to a coroutine function first.
    """
    if not asyncio.iscoroutinefunction(f):
        f = asyncio.coroutine(f)

    return asyncio.ensure_future(f(*args, **kwargs), loop=loop)



# Copyright (c) Facebook, Inc. and its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import logging
import six
import sys
import threading

UEXW_MAX_LENGTH = 1024


class TType:
    STOP = 0
    VOID = 1
    BOOL = 2
    BYTE = 3
    I08 = 3
    DOUBLE = 4
    I16 = 6
    I32 = 8
    I64 = 10
    STRING = 11
    UTF7 = 11
    STRUCT = 12
    MAP = 13
    SET = 14
    LIST = 15
    UTF8 = 16
    UTF16 = 17
    FLOAT = 19


class TMessageType:
    CALL = 1
    REPLY = 2
    EXCEPTION = 3
    ONEWAY = 4


class TPriority:
    """ apache::thrift::concurrency::PRIORITY """
    HIGH_IMPORTANT = 0
    HIGH = 1
    IMPORTANT = 2
    NORMAL = 3
    BEST_EFFORT = 4
    N_PRIORITIES = 5


class TRequestContext:
    def __init__(self):
        self._headers = None

    def getHeaders(self):
        return self._headers

    def setHeaders(self, headers):
        self._headers = headers


class TProcessorEventHandler:
    """Event handler for thrift processors"""
    # TODO: implement asyncComplete for Twisted

    def getHandlerContext(self, fn_name, server_context):
        """Called at the start of processing a handler method"""
        return None

    def preRead(self, handler_context, fn_name, args):
        """Called before the handler method's argument are read"""
        pass

    def postRead(self, handler_context, fn_name, args):
        """Called after the handler method's argument are read"""
        pass

    def preWrite(self, handler_context, fn_name, result):
        """Called before the handler method's results are written"""
        pass

    def postWrite(self, handler_context, fn_name, result):
        """Called after the handler method's results are written"""
        pass

    def handlerException(self, handler_context, fn_name, exception):
        """Called if (and only if) the handler threw an expected exception."""
        pass

    def handlerError(self, handler_context, fn_name, exception):
        """Called if (and only if) the handler threw an unexpected exception.

        Note that this method is NOT called if the handler threw an
        exception that is declared in the thrift service specification"""

        logging.exception("Unexpected error in service handler " + fn_name + ":")


class TServerInterface:
    def __init__(self):
        self._tl_request_context = threading.local()

    def setRequestContext(self, request_context):
        self._tl_request_context.ctx = request_context

    def getRequestContext(self):
        return self._tl_request_context.ctx


class TProcessor:

    """Base class for processor, which works on two streams."""

    def __init__(self):
        self._event_handler = TProcessorEventHandler()  # null object handler
        self._handler = None
        self._processMap = {}
        self._priorityMap = {}

    def setEventHandler(self, event_handler):
        self._event_handler = event_handler

    def getEventHandler(self):
        return self._event_handler

    def process(self, iprot, oprot, server_context=None):
        pass

    def onewayMethods(self):
        return ()

    def readMessageBegin(self, iprot):
        name, _, seqid = iprot.readMessageBegin()
        if six.PY3:
            name = name.decode('utf8')
        return name, seqid

    def skipMessageStruct(self, iprot):
        iprot.skip(TType.STRUCT)
        iprot.readMessageEnd()

    def doesKnowFunction(self, name):
        return name in self._processMap

    def callFunction(self, name, seqid, iprot, oprot, server_ctx):
        process_fn = self._processMap[name]
        return process_fn(self, seqid, iprot, oprot, server_ctx)

    def readArgs(self, iprot, handler_ctx, fn_name, argtype):
        args = argtype()
        self._event_handler.preRead(handler_ctx, fn_name, args)
        args.read(iprot)
        iprot.readMessageEnd()
        self._event_handler.postRead(handler_ctx, fn_name, args)
        return args

    def writeException(self, oprot, name, seqid, exc):
        oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
        exc.write(oprot)
        oprot.writeMessageEnd()
        oprot.trans.flush()

    def get_priority(self, fname):
        return self._priorityMap.get(fname, TPriority.NORMAL)

    def _getReplyType(self, result):
        if isinstance(result, TApplicationException):
            return TMessageType.EXCEPTION
        return TMessageType.REPLY

    @staticmethod
    def _get_exception_from_thrift_result(result):
        """Returns the wrapped exception, if pressent. None if not.

        result is a generated *_result object. This object either has a
        'success' field set indicating the call succeeded, or a field set
        indicating the exception thrown.
        """
        fields = (
            result.__dict__.keys()
            if hasattr(result, "__dict__") else result.__slots__
        )
        for field in fields:
            value = getattr(result, field)
            if value is None:
                continue
            elif field == 'success':
                return None
            else:
                return value
        return None

    def writeReply(self, oprot, handler_ctx, fn_name, seqid, result, server_ctx=None):
        self._event_handler.preWrite(handler_ctx, fn_name, result)
        reply_type = self._getReplyType(result)

        if server_ctx is not None and hasattr(server_ctx, 'context_data'):
            ex = (result if reply_type == TMessageType.EXCEPTION
                  else self._get_exception_from_thrift_result(result))
            if ex:
                server_ctx.context_data.setHeaderEx(ex.__class__.__name__)
                server_ctx.context_data.setHeaderExWhat(str(ex)[:UEXW_MAX_LENGTH])

        try:
            oprot.writeMessageBegin(fn_name, reply_type, seqid)
            result.write(oprot)
            oprot.writeMessageEnd()
            oprot.trans.flush()

        except Exception as e:
            # Handle any thrift serialization exceptions

            # Transport is likely in a messed up state. Some data may already have
            # been written and it may not be possible to recover. Doing nothing
            # causes the client to wait until the request times out. Try to
            # close the connection to trigger a quicker failure on client side
            oprot.trans.close()

            # Let application know that there has been an exception
            self._event_handler.handlerError(handler_ctx, fn_name, e)

            # We raise the exception again to avoid any further processing
            raise

        finally:
            # Since we called preWrite, we should also call postWrite to
            # allow application to properly log their requests.
            self._event_handler.postWrite(handler_ctx, fn_name, result)


class TException(Exception):

    """Base class for all thrift exceptions."""

    # BaseException.message is deprecated in Python v[2.6,3.0)
    if (2, 6, 0) <= sys.version_info < (3, 0):
        def _get_message(self):
            return self._message

        def _set_message(self, message):
            self._message = message

        message = property(_get_message, _set_message)

    def __init__(self, message=None):
        Exception.__init__(self, message)
        self.message = message


class TApplicationException(TException):

    """Application level thrift exceptions."""

    UNKNOWN = 0
    UNKNOWN_METHOD = 1
    INVALID_MESSAGE_TYPE = 2
    WRONG_METHOD_NAME = 3
    BAD_SEQUENCE_ID = 4
    MISSING_RESULT = 5
    INTERNAL_ERROR = 6
    PROTOCOL_ERROR = 7
    INVALID_TRANSFORM = 8
    INVALID_PROTOCOL = 9
    UNSUPPORTED_CLIENT_TYPE = 10
    LOADSHEDDING = 11
    TIMEOUT = 12
    INJECTED_FAILURE = 13

    EXTYPE_TO_STRING = {
        UNKNOWN_METHOD: 'Unknown method',
        INVALID_MESSAGE_TYPE: 'Invalid message type',
        WRONG_METHOD_NAME: 'Wrong method name',
        BAD_SEQUENCE_ID: 'Bad sequence ID',
        MISSING_RESULT: 'Missing result',
        INTERNAL_ERROR: 'Internal error',
        PROTOCOL_ERROR: 'Protocol error',
        INVALID_TRANSFORM: 'Invalid transform',
        INVALID_PROTOCOL: 'Invalid protocol',
        UNSUPPORTED_CLIENT_TYPE: 'Unsupported client type',
        LOADSHEDDING: 'Loadshedding request',
        TIMEOUT: 'Task timeout',
        INJECTED_FAILURE: 'Injected Failure',
    }

    def __init__(self, type=UNKNOWN, message=None):
        TException.__init__(self, message)
        self.type = type

    def __str__(self):
        if self.message:
            return self.message
        else:
            return self.EXTYPE_TO_STRING.get(
                self.type,
                'Default (unknown) TApplicationException')

    def read(self, iprot):
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    message = iprot.readString()
                    if sys.version_info.major >= 3 and isinstance(message,
                                                                  bytes):
                        try:
                            message = message.decode('utf-8')
                        except UnicodeDecodeError:
                            pass
                    self.message = message
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I32:
                    self.type = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        oprot.writeStructBegin(b'TApplicationException')
        if self.message is not None:
            oprot.writeFieldBegin(b'message', TType.STRING, 1)
            oprot.writeString(self.message.encode('utf-8')
                              if not isinstance(self.message, bytes)
                              else self.message)
            oprot.writeFieldEnd()
        if self.type is not None:
            oprot.writeFieldBegin(b'type', TType.I32, 2)
            oprot.writeI32(self.type)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()


class UnimplementedTypedef:
    pass



#!/usr/bin/env python
# --coding:utf-8--

# Copyright (c) 2020 vesoft inc. All rights reserved.
#
# This source code is licensed under Apache 2.0 License,
# attached with Common Clause Condition 1.0, found in the LICENSES directory.

from nebula2.common import ttypes
from nebula2.common.ttypes import Vertex, Tag, Edge
from nebula2.data.DataObject import (
    DataSetWrapper,
    Node,
    ValueWrapper,
    Relationship
)


class VertexData(object):
    PROP_START_INDEX = 1

    def __init__(self,
                 row,
                 col_names,
                 decode_type='utf-8'):
        """
        the format is
        '''
        |tag_name._vid|tag_name.prop1|tag_name.prop2|
        '''
        """
        if len(row.values) != len(col_names):
            raise RuntimeError(
                'Input row size is not equal with the col name size, {} != {}'
                    .format(row.values, col_names))
        self._row = row
        self._decode_type = decode_type
        self._col_names = []
        self._tag_name = ''
        for col_name in col_names:
            names = col_name.split(b'.')
            if len(names) != 2:
                raise RuntimeError('Input wrong col name format of tag')
            self._col_names.append(names[1])
            self._tag_name = names[0]

    def get_id(self):
        """get vertex id, if the space vid_type is int, you can use get_id().as_int(),
        if the space vid_type is fixed_string, you can use get_id().as_string()

        :return: ValueWrapper
        """
        if len(self._row.values) < 1:
            raise RuntimeError('The row value is bad format, '
                               'get vertex id failed: len is {}'
                               .format(len(self._row.values)))
        return ValueWrapper(self._row.values[0], self._decode_type)

    def as_node(self):
        """convert the vertex data to structure Node

        :return: Node
        """
        if len(self._row.values) < self.PROP_START_INDEX:
            raise RuntimeError('The row value is bad format, '
                               'as node failed: len is {}'
                               .format(len(self._row.values)))

        vertex = Vertex()
        vertex.tags = []
        vertex.vid = self._row.values[0]
        tag = Tag()
        tag.name = self._tag_name
        tag.props = {}
        index = self.PROP_START_INDEX
        while index < len(self._col_names):
            tag.props[self._col_names[index]] = self._row.values[index]
            index = index + 1
        vertex.tags.append(tag)

        return Node(vertex).set_decode_type(self._decode_type)

    def get_prop_values(self):
        """get all prop values from the vertex data

        :return: list<ValueWrapper>
        """
        index = self.PROP_START_INDEX
        prop_values = []
        while index < len(self._row.values):
            prop_values.append(ValueWrapper(self._row.values[index],
                                            decode_type=self._decode_type))
            index = index + 1
        return prop_values

    def __repr__(self):
        return str(self.as_node())


class EdgeData(object):
    PROP_START_INDEX = 4

    def __init__(self,
                 row,
                 col_names,
                 decode_type='utf-8'):
        """
        the format is
        '''
        |edge_name._src|edge_name._type|edge_name._rank|edge_name._dst|edge_name.prop1|edge_name.prop2|
        '''
        """
        if len(row.values) != len(col_names):
            raise RuntimeError('Input row size is not equal '
                               'with the col name size, {} != {}'
                               .format(len(row.values), len(col_names)))
        self._row = row
        self._decode_type = decode_type
        self._col_names = []
        self._edge_name = ''
        for col_name in col_names:
            names = col_name.split(b'.')
            if len(names) != 2:
                raise RuntimeError('Input wrong col name format of edge')
            self._col_names.append(names[1])
            self._edge_name = names[0]

    def get_src_id(self):
        """get src id, if the space vid_type is int, you can use get_src_id().as_int(),
        if the space vid_type is fixed_string, you can use get_src_id().as_string()

        :return: ValueWrapper
        """
        if len(self._row.values) < 1:
            raise RuntimeError('The row value is bad format, '
                               'get edge src id failed: len is {}'
                               .format(len(self._row.values)))
        return ValueWrapper(self._row.values[0], self._decode_type)

    def get_edge_name(self):
        """get edge name

        :return: edge name
        """
        return self._edge_name.decode(self._decode_type)

    def get_ranking(self):
        """get edge ranking

        :return: ranking
        """
        if len(self._row.values) < 3:
            raise RuntimeError('The row value is bad format, '
                               'get edge ranking failed: len is {}'
                               .format(len(self._row.values)))
        assert self._row.values[2].getType() == ttypes.Value.IVAL
        return self._row.values[2].get_iVal()

    def get_dst_id(self):
        """get dst id, if the space vid_type is int, you can use get_dst_id().as_int(),
        if the space vid_type is fixed_string, you can use get_dst_id().as_string()

        :return: ValueWrapper
        """
        if len(self._row.values) < 4:
            raise RuntimeError('The row value is bad format, '
                               'get edge dst id failed: len is {}'
                               .format(len(self._row.values)))
        return ValueWrapper(self._row.values[3], self._decode_type)

    def as_relationship(self):
        """convert the edge data to structure Relationship

        :return: Relationship
        """
        if len(self._row.values) < self.PROP_START_INDEX:
            raise RuntimeError('The row value is bad format, '
                               'as relationship failed: len is {}'
                               .format(len(self._row.values)))
        edge = Edge()
        edge.src = self._row.values[0]
        edge.type = self._row.values[1].get_iVal()
        edge.name = self._edge_name
        edge.ranking = self._row.values[2].get_iVal()
        edge.dst = self._row.values[3]
        edge.props = {}
        index = self.PROP_START_INDEX
        while index < len(self._col_names):
            edge.props[self._col_names[index]] = self._row.values[index]
            index = index + 1

        return Relationship(edge).set_decode_type(self._decode_type)

    def get_prop_values(self):
        """get all prop values from the edge data

        :return: list<ValueWrapper>
        """
        index = self.PROP_START_INDEX
        prop_values = []
        while index < len(self._row.values):
            prop_values.append(ValueWrapper(self._row.values[index], decode_type=self._decode_type))
            index = index + 1
        return prop_values

    def __repr__(self):
        return str(self.as_relationship())


class BaseResult(object):
    def __init__(self,
                 data_sets: list,
                 decode_type='utf-8',
                 is_vertex=True):
        assert data_sets is not None
        self.is_vertex = is_vertex
        self._data_sets = data_sets
        self._decode_type = decode_type
        self._pos = -1
        self._data_set_pos = 0
        self._table_pos = -1
        self._size = 0
        for data_set in self._data_sets:
            self._size += len(data_set.rows)

    def get_data_set(self):
        """get DataSet, it's the origin values, the string type is binary

        :return: DataSet
        """
        result = None
        for data_set in self._data_sets:
            if result is None:
                result = data_set
                continue
            if len(data_set.column_names) != len(result.column_names):
                raise RuntimeError('Multi DataSets are different col size')
            result.rows.extend(data_set.rows)

        return result

    def get_data_set_wrapper(self):
        """get DataSetWrapper, it's the wrapper for DataSet value, the string type is str

        :return: DataSetWrapper
        """
        result = None
        for data_set in self._data_sets:
            if result is None:
                result = data_set
                continue
            if len(data_set.column_names) != len(result.column_names):
                raise RuntimeError('Multi DataSets are different col size')
            result.rows.extend(data_set.rows)

        if result is None:
            return None
        return DataSetWrapper(result, self._decode_type)

    def __repr__(self):
        return str(self._data_sets)

    def __iter__(self):
        self._pos = -1
        return self

    def __next__(self):
        """The VertexData or EdgeData iterator

        :return: VertexData or EdgeData Iterator
        """
        if len(self._data_sets) == 0 or self._pos >= self._size - 1:
            raise StopIteration
        self._pos += 1
        self._table_pos += 1
        if self._table_pos >= len(self._data_sets[self._data_set_pos].rows):
            self._table_pos = 0
            self._data_set_pos += 1
        col_names = self._data_sets[self._data_set_pos].column_names
        row = self._data_sets[self._data_set_pos].rows[self._table_pos]
        if self.is_vertex:
            return VertexData(row, col_names, self._decode_type)
        else:
            return EdgeData(row, col_names, self._decode_type)




#!/usr/bin/env python
# --coding:utf-8--

# Copyright (c) 2020 vesoft inc. All rights reserved.
#
# This source code is licensed under Apache 2.0 License,
# attached with Common Clause Condition 1.0, found in the LICENSES directory.


class OutOfRangeException(Exception):
    def __init__(self):
        Exception.__init__(self)
        self.message = 'list index out of range'


class InvalidKeyException(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = "KeyError: `{}'".format(message)


class InvalidValueTypeException(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = "Invalid value type: `{}'".format(message)


class AuthFailedException(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = 'Auth failed: {}'.format(message)


class TagNotFoundException(Exception):
    def __init__(self, name):
        Exception.__init__(self)
        self.message = 'Tag:{} not found'.format(name)


class EdgeNotFoundException(Exception):
    def __init__(self, name):
        Exception.__init__(self)
        self.message = 'Edge:{} not found'.format(name)


class SpaceNotFoundException(Exception):
    def __init__(self, name):
        Exception.__init__(self, name)
        self.message = 'Space:{} not found'.format(name)


class PartNotFoundException(Exception):
    def __init__(self, part):
        Exception.__init__(self)
        self.message = 'Partition:{} not found'.format(part)


class NotValidConnectionException(Exception):
    def __init__(self):
        Exception.__init__(self)
        self.message = 'No extra connection'


class InValidHostname(Exception):
    def __init__(self, message):
        Exception.__init__(self, message)
        self.message = 'Invalid hostname: {}'.format(message)


class IOErrorException(Exception):
    E_UNKNOWN = 0
    E_ALL_BROKEN = 1
    E_CONNECT_BROKEN = 2
    E_TIMEOUT = 3
    E_NOT_OPEN = 4

    def __init__(self, code=E_UNKNOWN, message=None):
        Exception.__init__(self, message)
        self.type = code
        self.message = message

class ClientServerIncompatibleException(Exception):
    def __init__(self, message):
        Exception.__init__(self, f'Current client is not compatible with the remote server, please check the version: {message}')
